use axum::response::{
	IntoResponse,
	Redirect,
};
use axum::extract::{
	State,
	Query,
};
use openidconnect::{
	AuthenticationFlow,
	AuthorizationCode,
	CsrfToken,
	Nonce,
	OAuth2TokenResponse,
	Scope,
};
use openidconnect::core::CoreResponseType;
use openidconnect::reqwest::async_http_client;
use serde::Deserialize;

use crate::AppState;

#[derive(Deserialize)]
pub struct OidcCallback {
	code: String,
	state: String
}

pub async fn login(State(app_state): State<AppState>) -> impl IntoResponse {
	let (auth_url, csrf_state, nonce) = app_state.oidc_client.authorize_url(
		AuthenticationFlow::<CoreResponseType>::AuthorizationCode,
		CsrfToken::new_random,
		Nonce::new_random,
	)
		.add_scope(Scope::new("openid".to_string()))
		.add_scope(Scope::new("profile".to_string()))
		.url();
	let mut lock = app_state.state.lock().await;
	lock.insert(
		nonce.secret().to_owned(),
		csrf_state.secret().to_owned(),
	);
	Redirect::to(auth_url.as_str())
}

pub async fn callback(
	State(app_state): State<AppState>,
	Query(callback): Query<OidcCallback>,
) -> impl IntoResponse {
	println!("Received auth code: {}", callback.code.clone());
	let token_response = app_state.oidc_client
		.exchange_code(AuthorizationCode::new(callback.code.to_owned()))
		.request_async(async_http_client)
		.await
		.unwrap();
	println!("Exchanged for access token: {}", token_response.access_token().secret());
	println!("\tvalid for scopes {:?}", token_response.scopes());
	Redirect::to("/user/home")
//	let id_token_verifier: CoreIdTokenVerifier = app_state.oidc_client.id_token_verifier();
//	let id_token_claims: &CoreIdTokenClaims = token_response
//		.extra_fields()
//		.id_token()
//		.expect("IdP did not return an ID token")
//		.claims(&id_token_verifier, &nonce)
}
